local PitBull4 = _G.PitBull4
if not PitBull4 then
	error("PitBull4_ColorfulPowerBar requires PitBull4")
end

local L = PitBull4.L

local mod = PitBull4:NewModule("ColorfulPowerBar", "AceEvent-3.0")

mod:SetModuleType("bar")
mod:SetName(L["Colorful power bar"])
mod:SetDescription(L["Show a colorful power bar."])
mod.allow_animations = true
mod:SetDefaults({
	enabled = false,
	full_is_empty = false,
	threshold1 = 0.6,
	threshold2 = 0.3,
	threshold3 = 0.2,
	position = 4,
}, {
	powerbar_color_default = { 0, 1, 0 },
	powerbar_color1 = { 1, 1, 0 },
	powerbar_color2 = { 1, 0.5, 0 },
	powerbar_color3 = { 1, 0, 0 },
})

local guids_to_update = {}

local timerFrame = CreateFrame("Frame")
timerFrame:Hide()
timerFrame:SetScript("OnUpdate", function()
	if next(guids_to_update) then
		for frame in PitBull4:IterateFrames() do
			if guids_to_update[frame.guid] then
				mod:Update(frame)
			end
		end
		wipe(guids_to_update)
	end
end)

local function resetThresholds()
	for frame in PitBull4:IterateFrames() do
		frame.thresholds = nil
	end
end

function mod:OnEnable()
	self:RegisterEvent("UNIT_POWER_FREQUENT", "UNIT_POWER")
	self:RegisterEvent("UNIT_MAXPOWER", "UNIT_POWER")
	self:RegisterEvent("UNIT_DISPLAYPOWER", "UNIT_POWER")

	timerFrame:Show()
end

function mod:OnDisable()
	timerFrame:Hide()
	resetThresholds()
end


function mod:UNIT_POWER(_, unit)
	local guid = UnitGUID(unit)
	if guid then
		guids_to_update[guid] = true
	end
end


function mod:GetValue(frame)
	local unit = frame.unit
	local max = UnitPowerMax(unit)

	if max == 0 then
		return 0
	end

	return UnitPower(unit) / max
end

function mod:GetExampleValue(frame)
	return nil -- just use the current value
end

do
	local temp = {}
	function mod:GetColor(frame, value)
		local layout_db = self:GetLayoutDB(frame)
		local db = self.db.profile.global

		if not frame.thresholds then
			wipe(temp)
			for i=1, 3 do
				if layout_db["threshold"..i] > 0 then
					temp[#temp+1] = i
				end
			end
			table.sort(temp, function(a, b) return layout_db["threshold"..a] < layout_db["threshold"..b] end)
			frame.thresholds = temp
		end

		for i, v in ipairs(frame.thresholds) do
			local threshold, color = layout_db["threshold"..v], db["powerbar_color"..v]
			if value < threshold or (i == 1 and layout_db.full_is_empty and value == 1) then
				return unpack(color)
			end
		end

		return unpack(db.powerbar_color_default)
	end
end
mod.GetExampleColor = mod.GetColor

function mod:GetBackgroundColor(frame)
	local _, power_token = UnitPowerType(frame.unit)
	if not power_token then
		power_token = "MANA"
	end
	if power_token == "POWER_TYPE_RED_POWER" then
		power_token = "RAGE"
	end

	local color = PitBull4.PowerColors[power_token]
	if color then
		-- use a variance of the original resource color
		return (color[1] + 0.2)/3, (color[2] + 0.2)/3, (color[3] + 0.2)/3
	end
end


-- Settings

mod:SetColorOptionsFunction(function(self)
	local function get(info)
		return unpack(self.db.profile.global[info[#info]])
	end
	local function set(info, r, g, b)
		local color = self.db.profile.global[info[#info]]
		color[1], color[2], color[3] = r, g, b
		resetThresholds()
		self:UpdateAll()
	end
	return "powerbar_color1", {
		type = "color",
		name = L["Threshold 1 Color"],
		get = get,
		set = set,
	},
	"powerbar_color2", {
		type = "color",
		name = L["Threshold 2 Color"],
		get = get,
		set = set,
	},
	"powerbar_color3", {
		type = "color",
		name = L["Threshold 3 Color"],
		get = get,
		set = set,
	},
	"powerbar_color_default", {
		type = "color",
		name = L["Default Color"],
		desc = L["Color used when no thresholds are hit"],
		get = get,
		set = set,
		width = "full",
	},
	function(info)
		self.db.profile.global.powerbar_color_default = { 0, 1, 0 }
		self.db.profile.global.powerbar_color1 = { 1, 1, 0 }
		self.db.profile.global.powerbar_color2 = { 1, 0.5, 0 }
		self.db.profile.global.powerbar_color3 = { 1, 0, 0 }
	end
end)

mod:SetLayoutOptionsFunction(function(self)
	local function get(info)
		return PitBull4.Options.GetLayoutDB(self)[info[#info]]
	end
	local function set(info, value)
		local db = PitBull4.Options.GetLayoutDB(self)
		db[info[#info]] = value
		self.thresholds = nil
	end

	return "full_is_empty", {
		type = "toggle",
		name = L["Full as empty"],
		desc = L["Color 100% power as your lowest threshold."],
		get = get,
		set = set,
	},
	"threshold1", {
		type = "range", min = 0, max = 1, isPercent = true,
		name = L["Threshold 1"],
		desc = L["The power bar will use the corresponding color when the value is less than this amount. Use 0 to not use this color."],
		get = get,
		set = set,
	},
	"threshold2", {
		type = "range", min = 0, max = 1, isPercent = true,
		name = L["Threshold 2"],
		desc = L["The power bar will use the corresponding color when the value is less than this amount. Use 0 to not use this color."],
		get = get,
		set = set,
	},
	"threshold3", {
		type = "range", min = 0, max = 1, isPercent = true,
		name = L["Threshold 3"],
		desc = L["The power bar will use the corresponding color when the value is less than this amount. Use 0 to not use this color."],
		get = get,
		set = set,
	},
	"toggle_custom_color", nil,
	"custom_color", nil,
	"color_by_class", nil,
	"color_pvp_by_class", nil,
	"hostility_color", nil,
	"hostility_color_npcs", nil
end)

